package com.herou.demo;

import com.herou.lib.USDK;
import net.jflask.App;
import net.jflask.Route;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 类描述：　[演示一个简单的游戏服务业务（只有对接SDK功能）]<br/>
 * 项目名称：[java]<br/>
 * 包名：　　[com.herou.demo]<br/>
 * 创建人：　[杨挺]<br/>
 * 创建时间：[2021/7/19 下午5:58]<br/>
 */
public class GameSrv
{
	public static void main(String[] args) throws Exception
	{
		App app = new App(5001);
		app.scan(new GameSrv(app));
		app.servePath("/", "/");
		app.start();
		System.out.println("Listening on http://0.0.0.0:" + app.getPort());
	}

	private final App app;

	/** 游戏在SDK平台的应用码 */
	private static final String PRODUCT_ID = "获取方式请参考接入文档";

	/** 游戏的APP_KEY */
	private static final String PRODUCT_KEY = "获取方式请参考接入文档";
	/** SDK服务端为游戏的同步的秘钥 */
	private static final String CALLBACK_KEY = "获取方式请参考接入文档";

	private final static USDK usdk = new USDK(PRODUCT_ID, PRODUCT_KEY, CALLBACK_KEY, null);

	private final static Map<String, Map<String, Object>> GAME_USER = new HashMap<>();

	public GameSrv(App app)
	{
		this.app = app;
	}

	private static Map<String, Object> map(Object... key_val)
	{

		Map<String, Object> map = new HashMap<String, Object>();
		for (int i = 1; i < key_val.length; i += 2)
			if (key_val[i] != null && key_val[i - 1] != null)
				map.put(key_val[i - 1] + "", key_val[i]);
		return map;
	}

	/** 计算Data */
	private static String result(Object... key_val)
	{
		return new JSONObject(map(key_val)).toString();
	}

	@Route(value = "/login", method = "POST")
	public String login()
	{
		String cUid = app.getRequest().getForm("cUid");
		String cNameReq = app.getRequest().getForm("cName");
		String accessToken = app.getRequest().getForm("accessToken");
		JSONObject result = usdk.login(cUid, cNameReq, accessToken);
		System.out.println("USDK校验结果 :" + result);
		if (result != null && result.optInt("code", -1) == 0)
		{
			String uid = result.optString("cUid");
			String cName = result.optString("cName");
			String channel = result.optString("channel");
			int channelId = result.optInt("channelId");
			System.out.println(String.format("登录渠道：%s(%s)", channel, channelId));

			String gameUser = String.format("%s_%s", channelId, uid);
			Map<String, Object> userInfo = new HashMap<>();
			userInfo.put("channelId", channelId);
			userInfo.put("channelUserId", uid);
			userInfo.put("cName", cName);

			Map<String, Object> map = new HashMap<String, Object>();
			map.put("sdkuser", userInfo);
			GAME_USER.put(gameUser, map);
			return result("code", 0, "uid", gameUser, "channel", channel, "channelId", channelId);
		}
		return result("code", -1);
	}

	@Route(value = "/order", method = "POST")
	public String order()
	{
		String uid = app.getRequest().getForm("uid");
		String goodsId = app.getRequest().getForm("goodsId");
		return result("code", 0, "order", String.format("%s_%s_%s", uid, goodsId, System.currentTimeMillis() / 1000));
	}

	@Route(value = "/game_prop", method = "POST")
	/** 获取 道具ID */ public String gameProp()
	{
		List<Map> list = new ArrayList<>();
		list.add(map("key", "a01", "name", "苹果", "amount", 1.0));
		list.add(map("key", "a02", "name", "香蕉", "amount", 2.0));
		list.add(map("key", "a03", "name", "土豆", "amount", 3.0));
		list.add(map("key", "a04", "name", "草莓", "amount", 4.0));
		return result("code", 0, "msg", "获取成功(纯演示效果，模拟获取游戏内的档位)", "props", list);
	}

	@Route(value = "/payC", method = "POST")
	public String payCallback()
	{
		String data = app.getRequest().getForm("data");
		String sign = app.getRequest().getForm("sign");
		JSONObject result = usdk.payCallback(data, sign);
		if (result == null)
			return "FAIL";

		//		System.out.println(String.format("支付回调 gameOrder: %s\t游戏的订单号", result.optString("gameOrder")));
		//		System.out.println(String.format("支付回调 selfDefine: %s\t游戏的自定义信息", result.optString("selfDefine")));
		//
		//		System.out.println(String.format("支付回调 amount: %s\t付款金额", result.optString("amount")));
		//		System.out.println(String.format("支付回调 payTime: %s\t支付时间", result.optString("payTime")));
		//		System.out.println(String.format("支付回调 goodsId: %s\t支付的道具ID", result.optString("goodsId")));
		//		System.out.println(String.format("支付回调 channelUid: %s\t渠道用户ID", result.optString("channelUid")));
		//		System.out.println(String.format("支付回调 channel: %s\t渠道码", result.optString("channel")));
		//		System.out.println(String.format("支付回调 channelId: %s\t渠道ID", result.optString("channelId")));
		//		System.out.println(String.format("支付回调 orderNo: %s\tUSDK的订单号", result.optString("orderNo")));

		// TODO: 这里需要进行游戏订单的合法性校验
		// 1、渠道id是否一致
		// 2、用户信息是否一致
		// 3、自定义信息是否正确
		// 4、金额档位是否正确
		// 5、......

		// 模拟加档位
		gameOnPay(result);

		return "SUCCESS"; // SUCCESS (注意，只有这7个字符，不能有空白或其它字符）, 如果返回其它字符会定时重新回调
	}

	private void gameOnPay(JSONObject result)
	{
		String channelId = result.optString("channelId");
		String channelUid = result.optString("channelUid");
		String gameUser = String.format("%s_%s", channelId, channelUid);
		Map<String, Object> gameUserInfo = GAME_USER.get(gameUser);
		if (gameUserInfo == null)
			return;
		String goodsId = result.optString("goodsId");

		Map<String, Integer> pack = (Map<String, Integer>) gameUserInfo.get("pack");
		if (pack == null)
		{
			pack = new HashMap<>();
			gameUserInfo.put("pack", pack);
		}
		Integer num = pack.get(goodsId);
		if (num == null)
			num = 0;
		num++;
		pack.put(goodsId, num);
	}

	@Route(value = "/game_user", method = "POST")
	public String game_user()
	{
		String uid = app.getRequest().getForm("uid");
		if (GAME_USER.containsKey(uid))
		{
			return result("code", 0, "info", GAME_USER.get(uid));
		}
		return result("code", 1, "msg", "miss user");
	}

	@Route(value = "/game_user_free", method = "POST")
	public String game_user_free()
	{
		String uid = app.getRequest().getForm("uid");
		if (GAME_USER.containsKey(uid))
		{
			GAME_USER.remove(uid);
			return result("code", 0, "msg", "ok");
		}
		return result("code", 1, "msg", "miss user");
	}

}
